@@ -69,14 +69,30 @@ pub fn read_or_die(path: &Path) -> Vec<u8> {
 
 /// Do these bytes look like a binary (non-textual) format?
 pub fn is_probably_binary(bytes: &[u8]) -> bool {
-    // If more than 20 of the first 1,000 characters are not valid
-    // UTF-8, we assume it's binary.
+    let mime = tree_magic_mini::from_u8(bytes);
+    match mime {
+        // Treat pdf as binary.
+        "application/pdf" => return true,
+        // Treat all image content as binary.
+        v if v.starts_with("image/") => return true,
+        // Treat all audio content as binary.
+        v if v.starts_with("audio/") => return true,
+        // Treat all video content as binary.
+        v if v.starts_with("video/") => return true,
+        // Treat all font content as binary.
+        v if v.starts_with("font/") => return true,
+        _ => {}
+    }
+
+    // If more than 20 of the first 1,000 characters are null bytes or
+    // invalid UTF-8, we assume it's binary.
     let num_replaced = String::from_utf8_lossy(bytes)
         .to_string()
         .chars()
         .take(1000)
-        .filter(|c| *c == std::char::REPLACEMENT_CHARACTER)
+        .filter(|c| *c == std::char::REPLACEMENT_CHARACTER || *c == '\0')
         .count();
+
     num_replaced > 20
 }
 
@@ -111,7 +127,7 @@ pub fn relative_paths_in_either(lhs_dir: &Path, rhs_dir: &Path) -> Vec<PathBuf>
                 if !seen.contains(lhs_path) {
                     // It should be impossible to get duplicates, but
                     // be defensive.
-                    res.push(lhs_path.to_owned());
+                    res.push(lhs_path.clone());
                     seen.insert(lhs_path);
                 }
 
@@ -124,8 +140,8 @@ pub fn relative_paths_in_either(lhs_dir: &Path, rhs_dir: &Path) -> Vec<PathBuf>
                 } else if seen.contains(rhs_path) {
                     j += 1;
                 } else {
-                    res.push(lhs_path.to_owned());
-                    res.push(rhs_path.to_owned());
+                    res.push(lhs_path.clone());
+                    res.push(rhs_path.clone());
 
                     seen.insert(lhs_path);
                     seen.insert(rhs_path);
@@ -153,4 +169,9 @@ mod tests {
         let s = "hello world";
         assert!(!is_probably_binary(s.as_bytes()));
     }
+    #[test]
+    fn test_null_bytes_are_binary() {
+        let s = "\0".repeat(1000);
+        assert!(is_probably_binary(s.as_bytes()));
+    }
 }
