@@ -1,10 +1,12 @@
+//! CLI option parsing.
+
 use std::{borrow::Borrow, env, ffi::OsString};
 
 use atty::Stream;
 use clap::{crate_authors, crate_description, crate_version, Arg, Command};
 use const_format::formatcp;
 
-use crate::{guess_language, style::BackgroundColor};
+use crate::{display::style::BackgroundColor, parse::guess_language};
 
 pub const DEFAULT_NODE_LIMIT: u32 = 30_000;
 pub const DEFAULT_BYTE_LIMIT: usize = 1_000_000;
@@ -27,6 +29,7 @@ pub struct DisplayOptions {
     pub print_unchanged: bool,
     pub tab_width: usize,
     pub display_width: usize,
+    pub in_vcs: bool,
     pub syntax_highlight: bool,
 }
 
@@ -173,10 +176,17 @@ pub enum Mode {
         byte_limit: usize,
         display_options: DisplayOptions,
         missing_as_empty: bool,
-        display_path: String,
         language_override: Option<guess_language::Language>,
+        /// The path where we can read the LHS file. This is often a
+        /// temporary file generated by source control.
         lhs_path: OsString,
+        /// The path where we can read the RHS file. This is often a
+        /// temporary file generated by source control.
         rhs_path: OsString,
+        /// The path that we should display for the LHS file.
+        lhs_display_path: String,
+        /// The path that we should display for the RHS file.
+        rhs_display_path: String,
     },
     DumpTreeSitter {
         path: String,
@@ -225,29 +235,34 @@ pub fn parse_args() -> Mode {
     info!("CLI arguments: {:?}", args);
 
     // TODO: document these different ways of calling difftastic.
-    let (display_path, lhs_path, rhs_path) = match &args[..] {
+    let (lhs_display_path, rhs_display_path, lhs_path, rhs_path, in_vcs) = match &args[..] {
         [lhs_path, rhs_path] => (
+            lhs_path.to_owned(),
             rhs_path.to_owned(),
             lhs_path.to_owned(),
             rhs_path.to_owned(),
+            false,
         ),
         [display_path, lhs_tmp_file, _lhs_hash, _lhs_mode, rhs_tmp_file, _rhs_hash, _rhs_mode] => {
             // https://git-scm.com/docs/git#Documentation/git.txt-codeGITEXTERNALDIFFcode
             (
+                display_path.to_owned(),
                 display_path.to_owned(),
                 lhs_tmp_file.to_owned(),
                 rhs_tmp_file.to_owned(),
+                true,
             )
         }
-        [_old_name, lhs_tmp_file, _lhs_hash, _lhs_mode, rhs_tmp_file, _rhs_hash, _rhs_mode, new_name, _similarity] =>
+        [old_name, lhs_tmp_file, _lhs_hash, _lhs_mode, rhs_tmp_file, _rhs_hash, _rhs_mode, new_name, _similarity] =>
         {
             // Rename file.
-            // TODO: mention old name as well as diffing.
             // TODO: where does git document these 9 arguments?
             (
+                old_name.to_owned(),
                 new_name.to_owned(),
                 lhs_tmp_file.to_owned(),
                 rhs_tmp_file.to_owned(),
+                true,
             )
         }
         _ => {
@@ -340,6 +355,7 @@ pub fn parse_args() -> Mode {
         display_mode,
         display_width,
         syntax_highlight,
+        in_vcs,
     };
 
     Mode::Diff {
@@ -347,17 +363,18 @@ pub fn parse_args() -> Mode {
         byte_limit,
         display_options,
         missing_as_empty,
-        display_path: display_path.to_string_lossy().to_string(),
         language_override,
         lhs_path: lhs_path.to_owned(),
         rhs_path: rhs_path.to_owned(),
+        lhs_display_path: lhs_display_path.to_string_lossy().to_string(),
+        rhs_display_path: rhs_display_path.to_string_lossy().to_string(),
     }
 }
 
 /// Choose the display width: try to autodetect, or fall back to a
 /// sensible default.
 fn detect_display_width() -> usize {
-    term_size::dimensions().map(|(w, _)| w).unwrap_or(80)
+    term_size::dimensions().map_or(80, |(w, _)| w)
 }
 
 pub fn should_use_color(color_output: ColorOutput) -> bool {
